@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* converter.c - conversion dialog
 
-   Copyright (C) 1998 - 2001 Free Software Foundation
+   Copyright (C) 1998 - 2002 Free Software Foundation
 
    GHex is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -30,12 +30,12 @@
 
 static void conv_entry_cb(GtkEntry *, gint);
 static void get_cursor_val_cb(GtkButton *button, Converter *conv);
-static void set_values(Converter *conv, gulong val);
+static void set_values(Converter *conv, gulong val, guint group);
 
 Converter *converter = NULL;
 
 static gboolean
-is_char_ok(char c, gint base)
+is_char_ok(signed char c, gint base)
 {
 	/* ASCII which is base 0 is always ok */
 	if(base == 0)
@@ -107,6 +107,8 @@ entry_key_press_event_cb(GtkWidget *widget, GdkEventKey *event,
 	return FALSE;
 }
 
+#define BUFFER_LEN 40
+
 static GtkWidget *
 create_converter_entry(const gchar *name, GtkWidget *table,
 						GtkAccelGroup *accel_group, gint pos, gint base)
@@ -115,6 +117,8 @@ create_converter_entry(const gchar *name, GtkWidget *table,
     GtkWidget *entry;
     gint accel_key;
 
+    gchar str[BUFFER_LEN + 1];
+
 	/* label */
 	label = gtk_label_new(name);
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
@@ -134,6 +138,12 @@ create_converter_entry(const gchar *name, GtkWidget *table,
 							   GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
 	gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, pos, pos+1);
 	gtk_widget_show(entry);
+
+	if (GTK_IS_ACCESSIBLE (gtk_widget_get_accessible (entry))) {
+		g_snprintf (str, BUFFER_LEN, "Displays the value at cursor in %s", name+1);
+		add_atk_namedesc (entry, name+1, str);
+		add_atk_relation (entry, label, ATK_RELATION_LABELLED_BY);
+	}
        
 	return entry;
 }
@@ -158,26 +168,38 @@ create_converter_button(const gchar *name, GtkAccelGroup *accel_group)
 	return button;
 }    
 
+/* Made global. We need to access this in ui.c */
+GtkWidget *converter_get = NULL;
+
 static void
 close_converter(GtkWidget *button, GnomeDialog *dialog)
 {
 	gnome_dialog_close(dialog);
 }
 
+static gboolean
+converter_delete_event_cb(GtkWidget *widget, GdkEventAny *e, gpointer user_data)
+{
+	ghex_window_sync_converter_item(NULL, FALSE);
+	gtk_widget_hide(widget);
+	return TRUE;
+}
+
 Converter *
 create_converter()
 {
 	Converter *conv;
-	GtkWidget *table, *get;
+	GtkWidget *table;
 	GtkAccelGroup *accel_group;
+	gint i;
  
 	conv = g_new0(Converter, 1);
 
-	conv->window = gnome_dialog_new(_("GHex: Converter"),
+	conv->window = gnome_dialog_new(_("Base Converter"),
 									 GNOME_STOCK_BUTTON_CLOSE, NULL);
 	gnome_dialog_close_hides(GNOME_DIALOG(conv->window), TRUE);
 	gnome_dialog_button_connect(GNOME_DIALOG(conv->window), 0,
-								 close_converter, conv->window);
+								GTK_SIGNAL_FUNC(close_converter), conv->window);
 
 	table = gtk_table_new(6, 2, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), GNOME_PAD_SMALL);
@@ -192,35 +214,51 @@ create_converter()
 	accel_group = gtk_accel_group_new();
 
 	/* entries */
-	conv->entry[0] = create_converter_entry(_("_Binary"), table,
+	conv->entry[0] = create_converter_entry(_("_Binary:"), table,
 											 accel_group, 0, 2);
-	conv->entry[1] = create_converter_entry(_("_Octal"), table,
+	conv->entry[1] = create_converter_entry(_("_Octal:"), table,
 											 accel_group, 1, 8);
-	conv->entry[2] = create_converter_entry(_("_Decimal"), table,
+	conv->entry[2] = create_converter_entry(_("_Decimal:"), table,
 											 accel_group, 2, 10);
-	conv->entry[3] = create_converter_entry(_("_Hex"), table,
+	conv->entry[3] = create_converter_entry(_("_Hex:"), table,
 											 accel_group, 3, 16);
-	conv->entry[4] = create_converter_entry(_("_ASCII"), table,
+	conv->entry[4] = create_converter_entry(_("_ASCII:"), table,
 											 accel_group, 4, 0);
 
 	/* get cursor button */
-	get = create_converter_button(_("_Get cursor value"), accel_group);
+	converter_get = create_converter_button(_("_Get cursor value"), accel_group);
 
-	gtk_signal_connect(GTK_OBJECT(get), "clicked",
+	gtk_signal_connect(GTK_OBJECT(conv->window), "delete_event",
+						GTK_SIGNAL_FUNC(converter_delete_event_cb), conv);
+	gtk_signal_connect(GTK_OBJECT(converter_get), "clicked",
 						GTK_SIGNAL_FUNC(get_cursor_val_cb), conv);
-	gtk_table_attach_defaults(GTK_TABLE(table), get, 0, 2, 5, 6);
+	gtk_table_attach_defaults(GTK_TABLE(table), converter_get, 0, 2, 5, 6);
 
 	/* add the accelerators */
 	gtk_window_add_accel_group(GTK_WINDOW(conv->window), accel_group);
 
+	if (GTK_IS_ACCESSIBLE(gtk_widget_get_accessible(converter_get))) {
+		add_atk_namedesc (converter_get, _("Get cursor value"), _("Gets the value at cursor in binary, octal, decimal, hex and ASCII"));
+		for (i=0; i<5; i++) {
+			add_atk_relation (conv->entry[i], converter_get, ATK_RELATION_CONTROLLED_BY);
+			add_atk_relation (converter_get, conv->entry[i], ATK_RELATION_CONTROLLER_FOR);
+		}
+	}
+
 	return conv;
 }
 
 static void
 get_cursor_val_cb(GtkButton *button, Converter *conv)
 {
-	GtkWidget *view = gnome_mdi_get_active_view(mdi);
+	GtkWidget *view;
 	guint val, start;
+	GHexWindow *win = ghex_window_get_active();
+
+	if(win == NULL || win->gh == NULL)
+		return;
+
+	view = GTK_WIDGET(win->gh);
 
 	if(view) {
 		start = gtk_hex_get_cursor(GTK_HEX(view));
@@ -233,7 +271,7 @@ get_cursor_val_cb(GtkButton *button, Converter *conv)
 		} while((start % GTK_HEX(view)->group_type != 0) &&
 				(start < GTK_HEX(view)->document->file_size) );
 
-		set_values(conv, val);
+		set_values(conv, val, GTK_HEX(view)->group_type);
 	}
 }
 
@@ -248,11 +286,20 @@ clean(guchar *ptr)
 #define CONV_BUFFER_LEN 32
 
 static void
-set_values(Converter *conv, gulong val)
+set_values(Converter *conv, gulong val, guint group)
 {
 	guchar buffer[CONV_BUFFER_LEN + 1];
 	gint i;
 
+	if(group == 0) {
+		if(val > 0xFFFF)
+			group = 4;
+		else if(val > 0xFF)
+			group = 2;
+		else
+			group = 1;
+	}
+
 	conv->value = val;
 	
 	for(i = 0; i < 32; i++)
@@ -266,11 +313,14 @@ set_values(Converter *conv, gulong val)
 	g_snprintf(buffer, CONV_BUFFER_LEN, "%lu", val);
 	gtk_entry_set_text(GTK_ENTRY(conv->entry[2]), buffer);
 	
-	g_snprintf(buffer, CONV_BUFFER_LEN, "%08lX", val);
+	for(i = 0; i < group; i++) {
+		g_snprintf(buffer + 2*i, CONV_BUFFER_LEN - 2*i,
+				   "%02x", ((val & (0xFF << ((group - 1 - i)*8)))) >> (group - 1 - i)*8);
+	}
 	gtk_entry_set_text(GTK_ENTRY(conv->entry[3]), buffer);
 	
-	for(i = 0; i < 4; i++) {
-		buffer[i] =(val & (0xFF << (3 - i)*8)) >> (3 - i)*8;
+	for(i = 0; i < group; i++) {
+		buffer[i] = ((val & (0xFF << ((group - 1 - i)*8)))) >> (group - 1 - i)*8;
 		if(buffer[i] < ' ')
 			buffer[i] = '_';
 	}
@@ -282,7 +332,8 @@ static void
 conv_entry_cb(GtkEntry *entry, gint base)
 {
 	guchar buffer[33];
-	gchar *text, *endptr;
+	const gchar *text;
+	gchar *endptr;
 	gulong val = converter->value;
 	int i, len;
 	
@@ -329,5 +380,5 @@ conv_entry_cb(GtkEntry *entry, gint base)
 	if(val == converter->value)
 		return;
 	
-	set_values(converter, val);
+	set_values(converter, val, 0);
 }
